@@ -1,5 +1,4 @@
-require 'cgi'
-require 'erb'
+require 'action_view/erb/util'
 require 'set'
 
 module ActionView
@@ -98,7 +97,7 @@ module ActionView
       # Returns an escaped version of +html+ without affecting existing escaped entities.
       #
       # ==== Examples
-      #   escape_once("1 > 2 &amp; 3")
+      #   escape_once("1 < 2 &amp; 3")
       #   # => "1 &lt; 2 &amp; 3"
       #
       #   escape_once("&lt;&lt; Accept & Checkout")
