@@ -1,3 +1,4 @@
+# encoding: utf-8
 require 'abstract_unit'
 
 RequestMock = Struct.new("Request", :request_uri, :protocol, :host_with_port, :env)
@@ -28,6 +29,16 @@ class UrlHelperTest < ActionView::TestCase
     assert_equal "http://www.example.com?a=b&amp;c=d", url_for("http://www.example.com?a=b&amp;c=d")
   end
 
+  def test_url_for_with_back
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show", nil, nil, {'HTTP_REFERER' => 'http://www.example.com/referer'})
+    assert_equal 'http://www.example.com/referer', url_for(:back)
+  end
+
+  def test_url_for_with_back_and_no_referer
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show", nil, nil, {})
+    assert_equal 'javascript:history.back()', url_for(:back)
+  end
+
   # todo: missing test cases
   def test_button_to_with_straight_url
     assert_dom_equal "<form method=\"post\" action=\"http://www.example.com\" class=\"button-to\"><div><input type=\"submit\" value=\"Hello\" /></div></form>", button_to("Hello", "http://www.example.com")
@@ -80,7 +91,7 @@ class UrlHelperTest < ActionView::TestCase
   def test_link_tag_with_straight_url
     assert_dom_equal "<a href=\"http://www.example.com\">Hello</a>", link_to("Hello", "http://www.example.com")
   end
-  
+
   def test_link_tag_without_host_option
     ActionController::Base.class_eval { attr_accessor :url }
     url = {:controller => 'weblog', :action => 'show'}
@@ -121,7 +132,7 @@ class UrlHelperTest < ActionView::TestCase
     @controller.request = RequestMock.new("http://www.example.com/weblog/show", nil, nil, {'HTTP_REFERER' => 'http://www.example.com/referer'})
     assert_dom_equal "<a href=\"http://www.example.com/referer\">go back</a>", link_to('go back', :back)
   end
-  
+
   def test_link_tag_with_back_and_no_referer
     @controller.request = RequestMock.new("http://www.example.com/weblog/show", nil, nil, {})
     assert_dom_equal "<a href=\"javascript:history.back()\">go back</a>", link_to('go back', :back)
@@ -209,9 +220,17 @@ class UrlHelperTest < ActionView::TestCase
   end
 
   def test_link_tag_using_post_javascript_and_popup
-    assert_raises(ActionView::ActionViewError) { link_to("Hello", "http://www.example.com", :popup => true, :method => :post, :confirm => "Are you serious?") }
+    assert_raise(ActionView::ActionViewError) { link_to("Hello", "http://www.example.com", :popup => true, :method => :post, :confirm => "Are you serious?") }
   end
-  
+
+  def test_link_tag_using_block_in_erb
+    __in_erb_template = ''
+
+    link_to("http://example.com") { concat("Example site") }
+
+    assert_equal '<a href="http://example.com">Example site</a>', output_buffer
+  end
+
   def test_link_to_unless
     assert_equal "Showing", link_to_unless(true, "Showing", :action => "show", :controller => "weblog")
     assert_dom_equal "<a href=\"http://www.example.com\">Listing</a>", link_to_unless(false, "Listing", :action => "list", :controller => "weblog")
@@ -233,12 +252,55 @@ class UrlHelperTest < ActionView::TestCase
     assert_equal "Showing", link_to_if(false, "Showing", :action => "show", :controller => "weblog", :id => 1)
   end
 
+  def test_current_page_with_simple_url
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show")
+    @controller.url = "http://www.example.com/weblog/show"
+    assert current_page?({ :action => "show", :controller => "weblog" })
+    assert current_page?("http://www.example.com/weblog/show")
+  end
+  
+  def test_current_page_ignoring_params
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc&page=1")
+    @controller.url = "http://www.example.com/weblog/show?order=desc&page=1"
+    assert current_page?({ :action => "show", :controller => "weblog" })
+    assert current_page?("http://www.example.com/weblog/show")
+  end
+  
+  def test_current_page_with_params_that_match
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc&page=1")
+    @controller.url = "http://www.example.com/weblog/show?order=desc&page=1"
+    assert current_page?({ :action => "show", :controller => "weblog", :order => "desc", :page => "1" })
+    assert current_page?("http://www.example.com/weblog/show?order=desc&amp;page=1")
+  end
+  
   def test_link_unless_current
     @controller.request = RequestMock.new("http://www.example.com/weblog/show")
     @controller.url = "http://www.example.com/weblog/show"
     assert_equal "Showing", link_to_unless_current("Showing", { :action => "show", :controller => "weblog" })
     assert_equal "Showing", link_to_unless_current("Showing", "http://www.example.com/weblog/show")
 
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc")
+    @controller.url = "http://www.example.com/weblog/show"
+    assert_equal "Showing", link_to_unless_current("Showing", { :action => "show", :controller => "weblog" })
+    assert_equal "Showing", link_to_unless_current("Showing", "http://www.example.com/weblog/show")
+
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc&page=1")
+    @controller.url = "http://www.example.com/weblog/show?order=desc&page=1"
+    assert_equal "Showing", link_to_unless_current("Showing", { :action => "show", :controller => "weblog", :order=>'desc', :page=>'1' })
+    assert_equal "Showing", link_to_unless_current("Showing", "http://www.example.com/weblog/show?order=desc&amp;page=1")
+    assert_equal "Showing", link_to_unless_current("Showing", "http://www.example.com/weblog/show?order=desc&page=1")
+
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc")
+    @controller.url = "http://www.example.com/weblog/show?order=asc"
+    assert_equal "<a href=\"http://www.example.com/weblog/show?order=asc\">Showing</a>", link_to_unless_current("Showing", { :action => "show", :controller => "weblog" })
+    assert_equal "<a href=\"http://www.example.com/weblog/show?order=asc\">Showing</a>", link_to_unless_current("Showing", "http://www.example.com/weblog/show?order=asc")
+
+    @controller.request = RequestMock.new("http://www.example.com/weblog/show?order=desc&page=1")
+    @controller.url = "http://www.example.com/weblog/show?order=desc&page=2"
+    assert_equal "<a href=\"http://www.example.com/weblog/show?order=desc&amp;page=2\">Showing</a>", link_to_unless_current("Showing", { :action => "show", :controller => "weblog" })
+    assert_equal "<a href=\"http://www.example.com/weblog/show?order=desc&amp;page=2\">Showing</a>", link_to_unless_current("Showing", "http://www.example.com/weblog/show?order=desc&page=2")
+
+
     @controller.request = RequestMock.new("http://www.example.com/weblog/show")
     @controller.url = "http://www.example.com/weblog/list"
     assert_equal "<a href=\"http://www.example.com/weblog/list\">Listing</a>",
@@ -259,7 +321,11 @@ class UrlHelperTest < ActionView::TestCase
   end
 
   def test_mail_to_with_javascript
-    assert_dom_equal "<script type=\"text/javascript\">eval(unescape('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%6d%65%40%64%6f%6d%61%69%6e%2e%63%6f%6d%22%3e%4d%79%20%65%6d%61%69%6c%3c%2f%61%3e%27%29%3b'))</script>", mail_to("me@domain.com", "My email", :encode => "javascript")
+    assert_dom_equal "<script type=\"text/javascript\">eval(decodeURIComponent('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%6d%65%40%64%6f%6d%61%69%6e%2e%63%6f%6d%22%3e%4d%79%20%65%6d%61%69%6c%3c%2f%61%3e%27%29%3b'))</script>", mail_to("me@domain.com", "My email", :encode => "javascript")
+  end
+
+  def test_mail_to_with_javascript_unicode
+    assert_dom_equal "<script type=\"text/javascript\">eval(decodeURIComponent('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%75%6e%69%63%6f%64%65%40%65%78%61%6d%70%6c%65%2e%63%6f%6d%22%3e%c3%ba%6e%69%63%6f%64%65%3c%2f%61%3e%27%29%3b'))</script>", mail_to("unicode@example.com", "únicode", :encode => "javascript")
   end
 
   def test_mail_with_options
@@ -283,7 +349,8 @@ class UrlHelperTest < ActionView::TestCase
     assert_dom_equal "<a href=\"&#109;&#97;&#105;&#108;&#116;&#111;&#58;%6d%65@%64%6f%6d%61%69%6e.%63%6f%6d\">&#109;&#101;&#40;&#97;&#116;&#41;&#100;&#111;&#109;&#97;&#105;&#110;&#46;&#99;&#111;&#109;</a>", mail_to("me@domain.com", nil, :encode => "hex", :replace_at => "(at)")
     assert_dom_equal "<a href=\"&#109;&#97;&#105;&#108;&#116;&#111;&#58;%6d%65@%64%6f%6d%61%69%6e.%63%6f%6d\">My email</a>", mail_to("me@domain.com", "My email", :encode => "hex", :replace_at => "(at)")
     assert_dom_equal "<a href=\"&#109;&#97;&#105;&#108;&#116;&#111;&#58;%6d%65@%64%6f%6d%61%69%6e.%63%6f%6d\">&#109;&#101;&#40;&#97;&#116;&#41;&#100;&#111;&#109;&#97;&#105;&#110;&#40;&#100;&#111;&#116;&#41;&#99;&#111;&#109;</a>", mail_to("me@domain.com", nil, :encode => "hex", :replace_at => "(at)", :replace_dot => "(dot)")
-    assert_dom_equal "<script type=\"text/javascript\">eval(unescape('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%6d%65%40%64%6f%6d%61%69%6e%2e%63%6f%6d%22%3e%4d%79%20%65%6d%61%69%6c%3c%2f%61%3e%27%29%3b'))</script>", mail_to("me@domain.com", "My email", :encode => "javascript", :replace_at => "(at)", :replace_dot => "(dot)")
+    assert_dom_equal "<script type=\"text/javascript\">eval(decodeURIComponent('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%6d%65%40%64%6f%6d%61%69%6e%2e%63%6f%6d%22%3e%4d%79%20%65%6d%61%69%6c%3c%2f%61%3e%27%29%3b'))</script>", mail_to("me@domain.com", "My email", :encode => "javascript", :replace_at => "(at)", :replace_dot => "(dot)")
+    assert_dom_equal "<script type=\"text/javascript\">eval(decodeURIComponent('%64%6f%63%75%6d%65%6e%74%2e%77%72%69%74%65%28%27%3c%61%20%68%72%65%66%3d%22%6d%61%69%6c%74%6f%3a%6d%65%40%64%6f%6d%61%69%6e%2e%63%6f%6d%22%3e%6d%65%28%61%74%29%64%6f%6d%61%69%6e%28%64%6f%74%29%63%6f%6d%3c%2f%61%3e%27%29%3b'))</script>", mail_to("me@domain.com", nil, :encode => "javascript", :replace_at => "(at)", :replace_dot => "(dot)")
   end
   
   def protect_against_forgery?
@@ -293,8 +360,6 @@ end
 
 class UrlHelperWithControllerTest < ActionView::TestCase
   class UrlHelperController < ActionController::Base
-    self.view_paths = [ "#{File.dirname(__FILE__)}/../fixtures/" ]
-
     def self.controller_path; 'url_helper_with_controller' end
 
     def show_url_for
@@ -305,6 +370,10 @@ class UrlHelperWithControllerTest < ActionView::TestCase
       render :inline => "<%= show_named_route_#{params[:kind]} %>"
     end
 
+    def nil_url_for
+      render :inline => '<%= url_for(nil) %>'
+    end
+
     def rescue_action(e) raise e end
   end
 
@@ -321,7 +390,7 @@ class UrlHelperWithControllerTest < ActionView::TestCase
     assert_equal '/url_helper_with_controller/show_url_for', @response.body
   end
 
-  def test_named_route_shows_host_and_path
+  def test_named_route_url_shows_host_and_path
     with_url_helper_routing do
       get :show_named_route, :kind => 'url'
       assert_equal 'http://test.host/url_helper_with_controller/show_named_route', @response.body
@@ -335,6 +404,24 @@ class UrlHelperWithControllerTest < ActionView::TestCase
     end
   end
 
+  def test_url_for_nil_returns_current_path
+    get :nil_url_for
+    assert_equal '/url_helper_with_controller/nil_url_for', @response.body
+  end
+
+  def test_named_route_should_show_host_and_path_using_controller_default_url_options
+    class << @controller
+      def default_url_options(options = nil)
+        {:host => 'testtwo.host'}
+      end
+    end
+
+    with_url_helper_routing do
+      get :show_named_route, :kind => 'url'
+      assert_equal 'http://testtwo.host/url_helper_with_controller/show_named_route', @response.body
+    end
+  end
+
   protected
     def with_url_helper_routing
       with_routing do |set|
@@ -348,8 +435,6 @@ end
 
 class LinkToUnlessCurrentWithControllerTest < ActionView::TestCase
   class TasksController < ActionController::Base
-    self.view_paths = ["#{File.dirname(__FILE__)}/../fixtures/"]
-
     def self.controller_path; 'tasks' end
 
     def index
@@ -405,18 +490,17 @@ class LinkToUnlessCurrentWithControllerTest < ActionView::TestCase
     end
 end
 
-
 class Workshop
   attr_accessor :id, :new_record
 
   def initialize(id, new_record)
     @id, @new_record = id, new_record
   end
-  
+
   def new_record?
     @new_record
   end
-  
+
   def to_s
     id.to_s
   end
@@ -440,8 +524,6 @@ end
 
 class PolymorphicControllerTest < ActionView::TestCase
   class WorkshopsController < ActionController::Base
-    self.view_paths = ["#{File.dirname(__FILE__)}/../fixtures/"]
-
     def self.controller_path; 'workshops' end
 
     def index
@@ -458,8 +540,6 @@ class PolymorphicControllerTest < ActionView::TestCase
   end
 
   class SessionsController < ActionController::Base
-    self.view_paths = ["#{File.dirname(__FILE__)}/../fixtures/"]
-
     def self.controller_path; 'sessions' end
 
     def index
